/*start==============================================================================================================================================================================================
   Automatische Steuerung zum Maischen

   Überarbeitung "Bitter" (Lothar) und Borsti84 (Sebastian)

   V3.05FVHS - Funk für Display LCD 20x4 I2C mit Gradientenregelung


   Die Funksteckdosen beschriften (H->Heizung, R->Rühren, B->Brauerruf). Dann die Anlernfunktion der AiO aufrufen (SETUP -> FUNK) und die den jeweiligen
   Steckdosen zugeordnete Taste auf der Fernsteuerung drücken. Die Steckdosencodes werden im EEPROM abgelegt und stehen
   so immer zur Verfügung. Auf diese Art können sehr einfach ggf. neue Funksteckdosen angelernt werden.



   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
   Die Software wird unter der MIT - Lizenz veröffentlicht

   Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files ,
   to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   Hiermit wird unentgeltlich jeder Person, die eine Kopie der Software und der zugehörigen Dokumentationen erhält, die Erlaubnis erteilt,
   sie uneingeschränkt zu benutzen, inklusive und ohne Ausnahme dem Recht, sie zu verwenden, kopieren, ändern, fusionieren, verlegen, verbreiten, unterlizenzieren
   und/oder zu verkaufen, und Personen, die diese Software erhalten, diese Rechte zu geben, unter den folgenden Bedingungen:
   Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in allen Kopien oder Teilkopien der Software beizulegen.
   DIE SOFTWARE WIRD OHNE JEDE AUSDRÜCKLICHE ODER IMPLIZIERTE GARANTIE BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FÜR DEN VORGESEHENEN ODER
   EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTSVERLETZUNG, JEDOCH NICHT DARAUF BESCHRÄNKT. IN KEINEM FALL SIND DIE AUTOREN ODER COPYRIGHTINHABER FÜR JEGLICHEN
   SCHADEN ODER SONSTIGE ANSPRÜCHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFÜLLUNG EINES VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER
   SOFTWARE ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

  Über diese Version

    Alle Einstellungen sind mit dem Encoder vorzunehmen.
      - Encoder drehen = Werteverstellung
      - kurz Drücken = Eingabebestätigung
      - lang Drücken (> 4 Sekunden) = Abbruch und zurück zum Startmenü
      - Doppelklick innerhalb 0,6s führt in das Setupmenü
      - bei Abbruch werden alle eingegebenen Werte beibehalten
    Wenn man z.B. einen Fehler bei der Eingabe macht einfach lange drücken bis Abbruch. Dann die Eingaben durch drücken und drehen ggf. verändern.
      - während des automatischen Ablaufs sind durch Drehen des Encoders Änderungen an der Temperatur bzw. des Zeitablaufes möglich.

    Regler
    Die Steuerung der Heizung erfolgt über eine Gradientenregelung. Die Steigungsgeschwindigkeit des Temperaturanstiegs dT/dt bestimmt die Ein- und Ausschaltzeitpunkte. Sinnvoll für
    Einkocher ist hier ein Gradientenfaktor von "1".

    Sensorfehler
    Anzeige -127  => VCC (Versorgungsspannung des Sensors fehlt
    Anzeige 85.00   => Byte interner Sensorfehler ggf. Leitung zu lang (wird nicht überprüft!)
    Anzeige 0.00  => Datenleitung oder GND fehlt

    Der Rührer kann auf Dauerbetrieb oder Intervallbetrieb eingestellt werden. Prefix A = Ausschaltzeit – Prefix E = Einschaltzeit. Vorgabe jeweils 60s.
    Das Kochen des Suds kann erst nach einer vorgegebenen Temperaturschwelle erfolgen (Vorgabe 98°C).
    Es können max. 6 Hopfengaben festgelegt werden. Erinnert wird durch den Brauerruf.

    Die Funktionen die auf den Vergleich Sollwert mit Istwert
    beruhen werden immer erst nach 10 Messungen ausgeführt.

   Anschlüsse:

   PIN 4 : Taster von Encoder
   PIN 5 : DS1820 Sensor => 4,7k zw. 5V und PIN5
   PIN 8 : Drehencoder
   PIN 9 : Drehencoder
   PIN 10: Ruf (Summer) => Digitaler Ausgang
   PIN 12: Funksender
   PIN 2 : Funkempfänger (INT 0)
   PIN A4: Anschluss SDA vom Display
   PIN A5: Anschluss SCL vom Display


   Speicheraufteilung EEPROM ----------------------------------------------


   433 MHz - Heizung EIN [1]
   433 MHz - Heizung AUS [4]
   433 MHz - Rührer EIN [7]
   433 MHz - Rührer AUS [10]
   433 MHz - Ruf EIN [13]
   433 MHz - Ruf AUS [16]
   Rührer Dauerbetrieb J/N [19]
   Rührer Intervall EIN [20]
   Rührer Intervall AUS [21]
   Kochschwelle [22];
   Kühlzeitverzögerung [82]
   Gradientenfaktor (kH) [90]
   Einschaltverzögerung Heizrelais [94]
   Gradientenfaktor (kK) [98]
*/

//---------------------------------------------------------------------------
//Vorgaben
//---------------------------------------------------------------------------

//#define dbg       //Statusmeldungen werden im Serial-Monitor ausgegeben
//Bei Speicherplatzproblemen einkommentieren

//#define RuehrerMaischenAus    //Wenn der Rührer während des Einmaischens inaktiv sein soll,
//dann diese Direktive auskommentieren!

//EMV-Probleme -------------------------------------------------------------
//sporadischer -127 °C - Fehler durch elektromagnetische Einstreuungen (z.B. Zündtrafo, fehlerhafte Induktionsplatte)

#define emcSchutz               //einkommentieren, falls kein Problem!

#ifdef emcSchutz
long emccounter = 10000;
long emc;
#endif

//--------------------------------------------------------------------------

//---------------------------------------------------------------------------

#define HeizungEin_433 1
#define HeizungAus_433 4
#define RuehrerEin_433 7
#define RuehrerAus_433 10
#define RufEin_433 13
#define RufAus_433 16
#define RuehrerDauerbetrieb 19
#define RuehrerIntervallEin 20
#define RuehrerIntervallAus 21
#define Koch_Schwelle 22
#define Kuehlzeit_Verzoegerung 82
#define Gradienten_Faktor_Heizen 90
#define Einschaltverzoegerung_Heizung 94
#define Gradienten_Faktor_Kuehlen 98

//Funkcodierung-------------------------------------------------------------


unsigned long funk_heizung_ein;
unsigned long funk_heizung_aus;

unsigned long funk_ruehrer_ein;
unsigned long funk_ruehrer_aus;

//Brauerruf auf die Funksteckdose!
unsigned long funk_ruf_ein;
unsigned long funk_ruf_aus;

bool eingeschaltet = false;

byte adr = 1;  //Adresse im EPROM

#define RESEND_SNIFFED_VALUES 10             //jeden Befehl 10 mal wiederholen
unsigned long value;


//Ende Funkcodierung----------------------------------------------------------


//LCD Initialisierung --------------------------------------------------------

#include <LiquidCrystal_I2C.h>
// set the LCD address to 0x27 for a 20 chars 4 line display
// Set the pins on the I2C chip used for LCD connections:
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address

//----------------------------------------------------------------------------

//Click Encoder---------------------------------------------------------------

#include <ClickEncoder.h>
#include <TimerOne.h>

ClickEncoder *encoder;

void timerIsr() {
  encoder->service();
}

int last;
int wert;
bool ButtonPressed = false;
byte drehen = 0;            //Drehgeber Werte
byte tempdrehen;            //Merker für letzten Wert von drehen

//---------------------------------------------------------------------------



//Temperatursensor DS1820 Initialisierung -----------------------------------
#include <OneWire.h>
#include <DallasTemperature.h>

// Datenleitung an D5
#define ONE_WIRE_BUS 5

//OneWire-Instanz
OneWire oneWire(ONE_WIRE_BUS);

// Pass our oneWire reference to Dallas Temperature.
DallasTemperature sensors(&oneWire);

// arrays to hold device address
DeviceAddress insideThermometer;

//Funksender-----------------------------------------------------------------
#include <RCSwitch.h>
RCSwitch mySwitch = RCSwitch();
#define TransmitPin 12

//---------------------------------------------------------------------------

//Zeitmessung----------------------------------------------------------------
//#include <Time.h>  //Hier die richtige Bibliothek einbinden
#include <TimeLib.h>
//---------------------------------------------------------------------------

//EEPROM für Speicherwerte---------------------------------------------------
#include <EEPROM.h>
//---------------------------------------------------------------------------

//Ausgänge an DigitalPIN ----------------------------------------------------
#define schalterB  10                              //Zuordnung Braumeisterruf
//---------------------------------------------------------------------------

//Encoder Initialisierung ---------------------------------------------------

//Encoder drehen
#define pinA 8     //für Drehrichtungsänderung des Encoders PinA und PinB tauschen
#define pinB 9     //oder beliebige andere Ports angeben

#define ENC_DECODER (1 << 2)   //auskommentieren, wenn Probleme mit Encoder

//#define ENC_HALFSTEP 1     //auskommentieren, wenn Probleme mit Encoder

#define stepsPerNotch 4 // Klicks bis zum nächsten Schritt

// Taster drücken
#define taster 4     // Taster am Pin 4 für Bestätigung und Abbruch


//Allgemein Initialisierung--------------------------------------------------
byte anfang = 0;
float altsekunden;
byte regelung = 0;
byte heizung = 0;
byte sensorfehler = 0;
boolean sollwerterreicht = false;
//long kwarten = 0;                                     //Initialisierung Wartezeit kühlen - nicht ändern!
float sensorwert;
float isttemp = 20;                                   //Vorgabe 20 damit Sensorfehler am Anfang nicht anspricht
int modus = 0;
byte altmodus;
int rufmodus = 0;
float rufsignalzeit = 0;
boolean nachgussruf = false;                          //Signal wenn Nachgusstemp erreicht
byte x = 1;                                           //aktuelle Rast Nummer
byte h = 0;                                           //aktuelle Hopfengabe
byte y = 1;                                           //Übergabewert von x für Braumeisterruf
byte n = 0;                                           //Counter Messungserhöhung zur Fehlervermeidung
long int m = 0;                                       //EMC - Fehlerzähler
byte pause = 0;                                       //Counter für Ruftonanzahl

byte sekunden = 0;                                    //Zeitzählung
byte minuten = 0;                                     //Zeitzählung
byte minutenwert = 0;                                 //Zeitzählung
byte stunden = 0;                                     //Zeitzählung
byte blinker = 0;                                     // Hilfsvariable zum Blinken "Warten"


//Vorgabewerte zur ersten Einstellung-------------------------------------------
byte sollwert = 20;                                   //Sollwertvorgabe für Anzeige
byte maischtemp = 45;                                 //Vorgabe Einmasichtemperatur
byte rasten = 3;                                      //Vorgabe Anzahl Rasten
byte rastTemp[] = {
  0, 55, 64, 72, 72, 72
};        //Rasttemperatur Werte
byte rastZeit[] = {
  0, 15, 40, 25, 20, 20
};              //Rastzeit Werte
byte braumeister[] = {
  0, 0, 0, 0, 0, 0
};               //Braumeisterruf standart AUS
byte endtemp = 78;                                   //Vorgabewert Endtemperatur
byte kochzeit = 90;                                  //90 min. Vorgabe
byte abbruchkochen;


/* Hopfengaben--------------------------------------------------------------------

   "maxrasten" kann geändert werden. die Vorgabewerte in den Klammern müssen nicht, können
    aber angepasst werden. "hanz" ist die Voreinstellung für die Auswahl der Rastanzahl.
*/

byte const maxgaben = 6;
byte hopfengabe[maxgaben] = {0, 0, 0, 0, 0, 0};     //Zeiten für die Hopfengaben 90 min. können vorgegeben werden
//byte hopfengabe[maxgaben] = (90,20,10,0,0,0];
byte hanz = 3; //Vorgabewert

// Erweiterung Rührwerk-----------------------------------------------------------

char ja_nein[2] = {'N', 'J'};                       //Antworten Dauerbetrieb Ja/nein
byte intervalEin = 60;                              //Voreinstellung 60s EIN
byte intervalAus = 60;                              // dann 60s AUS
char dauerbetrieb = 'N';                            //Voreinstellung Intervallbetrieb
byte altdrehen;
bool firstinterval = true;                          //erstes Intervall nach Heizphase
long intervalstart;
byte rwerk = 0;                                     //Hilfsvariable zur Rührwerkssteuerung


//Kochen--------------------------------------------------------------------------
bool schwelle_erreicht = false;

byte kschwelle = 98;                                //Voreinstellung - ab hier ertönt der Brauerruf


// Kühlen Einschaltverzögerung im ms - wird nach dem 1. Anlauf auf 300000 (5 min.) gesetzt---------

unsigned long kzeitverz = 0;                        //Init Wartezeit Einschalten Kompressor Vorgabe 5 min.

//Steigungsberechnung K/m -> Kelvin/Minute - Hinweis auf die Aufheizrate des Kessels

float alte_zeit;                                    //Zeit bei t-1
float alte_temperatur;                              //Temperatur bei t-1
unsigned long messzyklus = 10000;                   // Zeit im ms für dT und dt Vorgabe 60s

float gradient = 0; // Steigungswert
float kh = 1; //Gradientenfaktor kh (Heizen) - Vorgabe 1
float kk = 1; //Gradientenfaktor kk (Kühlen) - Vorgabe 1

long hwarten =  0;  //Wartezeitmessung Start
long esvheizen = 0; //Bei Gasbetrieb 10000ms (esvheizen = 10000) Einschaltverzögerung für Zündung
bool first = true;  // beim ersten Durchlauf keine Einschaltverzögerung Heizen/Kühlen

bool dclick = false;  //Merker ob Doppelclick betätigt wurde

byte zeitz = true;   //nach Doppelclick den Zeitzähler nicht neu starten wenn zeitz = false

unsigned long heizungInterval = 0; //Überprüfung ob Funksignal gesendet wird

//Jodprobe-------------------------------------------------------------------------

int tz = 0;                       //Anzahl Piepser für Jodprobe - Vorgabe 3 in Modus 29
bool signalende;                  //nur einmal an Jodprobe erinnern
bool jodprobe;                    //löscht die letzte Zeile mit dem Schriftzug "Jodprobe"

//setup============================================================================
void setup()
{

#ifdef debug
  Serial.begin(115200);
  Serial.print(EEPROM.read(HeizungEin_433));
  Serial.print(EEPROM.read(HeizungAus_433));
  Serial.print(EEPROM.read(RuehrerEin_433));
  Serial.print(EEPROM.read(RuehrerAus_433));
  Serial.print(EEPROM.read(RufEin_433));
  Serial.print(EEPROM.read(RufAus_433));
#endif

  drehen = sollwert;

  encoder = new ClickEncoder(pinB, pinA, taster, stepsPerNotch);

  //Stabilisierung des Drehencoders

  digitalWrite(pinA, HIGH);  //PullUp-Widerstand einschalten
  digitalWrite(pinB, HIGH);  //PullUp-Widerstand einschalten

  Timer1.initialize(1000);
  Timer1.attachInterrupt(timerIsr);

  last = -1;

  mySwitch.enableTransmit(TransmitPin);
  mySwitch.setRepeatTransmit(RESEND_SNIFFED_VALUES);


  pinMode(schalterB, OUTPUT);

  if (EEPROMReadlong(Gradienten_Faktor_Heizen ) > 100) EEPROMWritelong(Gradienten_Faktor_Heizen , kh * 10); //Vorgabewert Gradientenfaktor
  kh = EEPROMReadlong(Gradienten_Faktor_Heizen ) / 10;
  if (EEPROMReadlong(Gradienten_Faktor_Kuehlen) > 100) EEPROMWritelong(Gradienten_Faktor_Kuehlen, kk * 10); //Vorgabewert Gradientenfaktor
  kk = EEPROMReadlong(Gradienten_Faktor_Kuehlen) / 10;
  esvheizen = EEPROMReadlong(Einschaltverzoegerung_Heizung);
  if (EEPROMReadlong(Einschaltverzoegerung_Heizung) > 60000) EEPROMWritelong(Einschaltverzoegerung_Heizung, 10000);  //10s falls noch nicht gesetzt

  kzeitverz = EEPROMReadlong(Kuehlzeit_Verzoegerung);
  if (EEPROMReadlong(Kuehlzeit_Verzoegerung) > 1000000) EEPROMWritelong(Kuehlzeit_Verzoegerung, 300000);  //300s falls noch nicht gesetzt



  //LCD Setup ---------------------------------------------------------------------------------
  lcd.begin(20, 4);  // Initialisierung LCD

  lcd.setCursor(0, 0);
  lcd.print("AiO_NANO_F_04.02.2018");

  delay (2000);


  x = 1;

  lcd.clear();
  // ------------------------------------------------------------------------------------------


  //Temperatursensor DS1820 -------------------------------------------------------------------
  sensors.getAddress(insideThermometer, 0);
  // set the resolution to 9 bit (Each Dallas/Maxim device is capable of several different resolutions)
  sensors.setResolution(insideThermometer, 12);
  // ------------------------------------------------------------------------------------------

  //Rührparameter einlesen --------------------------------------------------------------------

  if (EEPROM.read(RuehrerDauerbetrieb) == 0) dauerbetrieb = 'N';
  if (EEPROM.read(RuehrerDauerbetrieb) == 1) dauerbetrieb = 'J';

  intervalEin = EEPROM.read(RuehrerIntervallEin);
  intervalAus = EEPROM.read(RuehrerIntervallAus);

  if (intervalEin == 255) (intervalEin = 60);  //wenn noch nichts im EEPROM
  if (intervalAus == 255) (intervalAus = 60);

  // Kochschwelle einlesen --------------------------------------------------------------------
  kschwelle = EEPROM.read(Koch_Schwelle);

  if (kschwelle == 255) (kschwelle = 97);  //wenn noch nichts im EEPROM
  //-------------------------------------------------------------------------------------------

  //Funksteckdosencode einlesen ---------------------------------------------------------------

  init_funk();

  //-------------------------------------------------------------------------------------------

  //Funksteckdosen Heizung/Rührer Aus----------------------------------------------------------

  mySwitch.send(funk_heizung_aus, 24); //Schaltet die FUnksteckdose aus
  delay(100);
  rwerk = 1;
  ruehrerEinAus(0);                   //zunächst natürlich AUS

  //für Gradientenberechnung -------------------------------------------------------------------

  alte_zeit = millis();
  alte_temperatur = isttemp;

}



//loop==========================================================================================
void loop()
{
  drehen += encoder->getValue();

  if (drehen != last) {
    last = drehen;
  }

  ButtonPressed = false;
  ClickEncoder::Button b = encoder->getButton();
  if (b != ClickEncoder::Open)
  {
    switch (b)
    {
      case ClickEncoder::Pressed:
        ButtonPressed = true;
        break;
      case ClickEncoder::Held:
        modus = 80;
        break;
      case ClickEncoder::Released:
        ButtonPressed = false;
        break;
      case ClickEncoder::Clicked:
        ButtonPressed = true;
        break;
      case ClickEncoder::DoubleClicked:
        altmodus = modus; //Merker für letzten Modus
        tempdrehen = drehen; //Merker für aktuellen Wert von drehen
        anfang = 0;
        dclick = true;  //Doppelclick wurde betätigt
        modus = 5;
        zeitz = false;
        break;
    }
  }

  if ((modus == 1) || (modus == 2) || ((modus >= 24) && (modus <= 42)))
  {
    // Zeitermittlung ---------------------------------------------------------------------------
    sekunden = second();  //aktuell Sekunde abspeichern für die Zeitrechnung
    minutenwert = minute(); //aktuell Minute abspeichern für die Zeitrechnung
    stunden = hour();     //aktuell Stunde abspeichern für die Zeitrechnung
    // ------------------------------------------------------------------------------------------

    // Ständige Überprüfung ob Schaltbefehl ausgeführt ------------------------------------------
    if (heizung == 1)
    {
      if ((heizungInterval + 5000) < millis())
      {
        heizungInterval = millis();
        mySwitch.send(funk_heizung_ein, 24);
        rwerk = 0;
        ruehrerEinAus(1);
      }
    }

    if (heizung == 0)
    {
      if ((heizungInterval + 5000) < millis())
      {
        heizungInterval = millis();
        mySwitch.send(funk_heizung_aus, 24);
      }
    }
    // ------------------------------------------------------------------------------------------

  }// if modus


#ifdef emcSchutz
  sensorwert = sensors.getTempC(insideThermometer);
  m = 0;
  while ((int(sensorwert) == -127) && (m < emccounter))
  {
    sensorwert = sensors.getTempC(insideThermometer);
    m++;
  }

  if ((sensorwert != isttemp) && (n < 3)) // Messfehlervermeidung
  { // des Sensorwertes
    n++;                                   // nach mehreren
  }                                        // Messungen
  else
  {
    isttemp = sensorwert;
    n = 0;
  }
#else
  sensorwert = sensors.getTempC(insideThermometer);


  if ((sensorwert != isttemp) && (n < 3)) // Messfehlervermeidung
  { // des Sensorwertes
    n++;                                   // nach mehreren
  }                                        // Messungen
  else
  {
    isttemp = sensorwert;
    n = 0;
  }

#endif
  //---------------------------------------------------------------------------------------------


  // Sensorfehler -------------------------------------------------------------------------------
  // Sensorfehler -127 => VCC fehlt
  // Sensorfehler 85.00 => interner Sensorfehler ggf. Leitung zu lang
  //                       nicht aktiviert


  if (regelung == 1) //nur bei Modi mit Regelung
  {
    if (int(sensorwert) == -127) {
      //zur besseren Erkennung Umwandling in (int)-Wert
      //sonst Probleme mit der Erkennung gerade bei 0.00
      if (sensorfehler == 0)
      {
        rufmodus = modus;
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Sensorfehler");
        lcd.setCursor(0, 1);
        lcd.print("Sensor ueberpruefen!");
        digitalWrite(schalterB, HIGH);
        delay (2000);
        digitalWrite(schalterB, LOW);
        regelung = 0;
        heizung = 0;
        rwerk = 1;
        ruehrerEinAus(0);
#ifdef emcSchutz
        emc = emccounter;
        emccounter = 1;
#endif
        sensorfehler = 1;
        modus = 31;
      }
    } else sensorfehler = 0;
  }
  //-----------------------------------------------------------------------------------------------


  // Temperaturanzeige Istwert --------------------------------------------------------------------
  lcd.setCursor(7, 3);
  lcd.print("ist ");
  lcd.print(float(sensorwert), 1);
  lcd.print(char(0xDF));
  if (int(sensorwert) != -127) lcd.print("C   "); else lcd.print("C ");

  // -------------------------------------------------------------------------------------------



  //Heizregelung----------------------------------------------------------------------------------
  if (regelung == 1)
  {
    // Temperaturanzeige Sollwert ---------------------------------------
    lcd.setCursor(15, 1);
    lcd.print(int(sollwert), 1);
    lcd.print(char(0xDF)); lcd.print("C ");

    // ----------------------------------------------------------------------------------------------

    if (modus == 0) alte_zeit = millis();

    if ((modus == 20) || (modus == 27) || (modus == 28) || (modus == 29) || (modus == 30) || (modus == 2) || (modus == 1))  //Maischeautomatik/Nachguss/Kühlen
    {
      lcd.setCursor(0, 0);
      if ((modus != 2) && (modus != 1)) lcd.print("Maischen");
      if ((alte_zeit + messzyklus) < millis()) {
        gradient = (isttemp - alte_temperatur) / (millis() - alte_zeit) * 60000;

        alte_zeit = millis();
        alte_temperatur = isttemp;
      }
      lcd.setCursor(0, 1);
      lcd.print(gradient);
      lcd.print(" K/m  ");
    }

    //Gradientenregelung ------------------------------------------------------------------------

    if (modus != 1) {
      if (isttemp > (sollwert - (gradient * kh))) {
        if (heizung == 1) {
          heizung = 0;
          hwarten = millis();
          sollwerterreicht = true;
        }

      }
      if (isttemp <= (sollwert - (gradient * kh))) {
        if ((millis() > (hwarten + esvheizen)) || first) {
          first = false;
          if (isttemp <= sollwert) heizung = 1;
        }
        sollwerterreicht = false;
      }
    }

    if (modus == 1) {  //Kühlregelung
      if (isttemp < (sollwert - (gradient * kk))) {
        if (heizung == 1) {
          heizung = 0;
          hwarten = millis();
        }

      }
      if (isttemp >= (sollwert - (gradient * kk))) {
        if ((millis() > (hwarten + kzeitverz)) || first) {
          first = false;
          if (isttemp >= sollwert) heizung = 1;
        }
      }
    }
  }
  // ------------------------------------------------------------------------------------------

  // Zeigt "Heizen" wenn Heizen und "Kühlen wenn Kühlen
  // und schalter die Pins und Funksteckdosen--------------------------------------------------
  if (heizung == 1)
  {
    lcd.setCursor(0, 3);
    if (modus != 1) {
      lcd.print("Heizen");
    } else  {
      lcd.print("K");
      lcd.print(char(0xF5));
      lcd.print("hlen");
    }


    if (!eingeschaltet)
    { mySwitch.send(funk_heizung_ein, 24); //nicht bei jedem Durchlauf aufrufen
      delay(100);
      eingeschaltet = true;
    }
    if (!sollwerterreicht) ruehrerEinAus(1);
    if (modus != 1) if (rwerk == 0) ruehrerEinAus(1);  //nicht im Kühlmodus
  }
  else
  {

    if (heizung == 0) {
      lcd.setCursor(0, 3);
      lcd.print("      ");
    }
    if (eingeschaltet)
    { mySwitch.send(funk_heizung_aus, 24); //nicht bei jedem Durchlauf aufrufen
      delay(100);
      eingeschaltet = false;
    }
    if (modus != 1)
    { //nicht im Kühlmodus
      if ((dauerbetrieb == 'J') || (modus == 27)) {
        rwerk = 0; // alter Status des Rührers -> notwendig wegen Funksteuerung
        // damit nicht fortwährend Daten gesendet werden!
        ruehrerEinAus(1);
      } //Rührer im Dauerbetrieb
      else if ((heizung == 0) && (regelung == 1) && (modus != 2) && (sollwerterreicht)) intervallbetrieb(); //Rührer im Intervallbetrieb
    }
  }
  //Ende Heizregelung--------------------------------------------------------------------------


  // Abfrage Modus

  regelung = 0;

  switch (modus) {
    case 0: {
        //Hauptmenü
        funktion_hauptschirm();
        break;
      }
    case 1: {
        regelung = 1;  //Kühlen
        funktion_temperatur();
        break;
      }
    case 2: {
        regelung = 1;  //Nachguss
        funktion_temperatur();
        break;
      }
    case 5: {
        //Einstellungen
        funktion_SetupMenu();
        break;
      }
    case 11: {
        //Gradientenfaktor kh für Heizen
        kDHWert();
        break;
      }

    case 12: {
        //Einschaltverzögerung Heizrelais
        ESVHeiz();
        break;
      }

    case 13: {
        //Gradientenfaktor kk für Kühlen
        kDKWert();
        break;
      }

    case 19: {
        //Anzahl Rasten
        funktion_rastanzahl();
        break;
      }
    case 20: {
        //Eingabe Einmaischtemperatur
        funktion_maischtemperatur();
        break;
      }
    case 21: {
        //Eingabe der Temperatur der Rasten
        funktion_rasteingabe();
        break;
      }
    case 22: {
        //Eingabe der Rastzeitwerte
        funktion_zeiteingabe();
        break;
      }
    case 23: {
        //Rührwerk Dauerlauf?
        ruehrwerk();
        break;
      }
    case 231: {
        //Rührwerk Intervall EIN
        ruehrwerk_ein();
        break;
      }
    case 232: {
        //Rührwerk Intervall AUS
        ruehrwerk_aus();
        break;
      }
    case 233: {
        //Steckdosencode eingeben
        steckdosencode();
        break;
      }

    case 14: {
        //Wartezeit bein Kühlen vor dem Wiedereinschalten
        wartenKuehlen();
        break;
      }
    case 239: {
        //Kochschwelle - ab hier Brauerruf
        kochschwelle();
        break;
      }
    case 24: {
        //Eingabe Braumeisterruf An/Aus
        funktion_braumeister();
        break;
      }
    case 25: {
        //Abmaischtemperatur
        funktion_endtempeingabe();
        break;
      }
    case 26: {
        //Start Maischeprozess
        funktion_startabfrage();
        break;
      }
    case 27: {
        regelung = 1;  //Automatischer Ablauf Maischen
        funktion_maischtemperaturautomatik();
        break;
      }
    case 28: {
        regelung = 1;
        funktion_tempautomatik();
        break;
      }
    case 29: {
        regelung = 1;
        funktion_zeitautomatik();
        break;
      }
    case 30: {
        regelung = 1;
        funktion_endtempautomatik();
        break;
      }
    case 31: {

        funktion_braumeisterrufalarm();
        break;
      }
    case 32: {

        funktion_braumeisterruf();
        break;
      }
    case 38: {

        anzahl_hopfengaben();
        break;
      }
    case 39: {

        funktion_hopfengabe();
        break;
      }
    case 40: {

        funktion_kochzeit();
        break;
      }
    case 41: {

        funktion_startkochen();
        break;
      }
    case 42: {

        funktion_kochen();
        break;
      }
    case 80: {

        funktion_abbruch();
        break;
      }
  }
}
// Ende Loop
// ------------------------------------------------------------------------------------------


//Funktionen===================================================================================
// ab hier Funktionen


// Funktion Hauptschirm------------------------------------------------------------------------
void funktion_hauptschirm()      //Modus=0
{
  if ((anfang == 0) || (dclick))

  {
    lcd.clear();
    lcd.setCursor(2, 0);
    lcd.print("Maischen");
    lcd.setCursor(2, 1);
    lcd.print("Nachguss");
    lcd.setCursor(2, 2);
    lcd.print("Kochen");
    lcd.setCursor(14, 0);
    lcd.print("K"); lcd.print(char(0xF5)); lcd.print("hlen");
    lcd.setCursor(14, 1);
    lcd.print("Setup");
    drehen = 0;
    altdrehen = 0;
    anfang = 1;
    dclick = false;
  }


  if (drehen == 255)
  {
    drehen = 4;
  }
  if (drehen == 5)
  {
    drehen = 0;
  }

  if (drehen != altdrehen)
  {
    lcd.setCursor(0, 0);
    lcd.print("  ");
    lcd.setCursor(0, 1);
    lcd.print("  ");
    lcd.setCursor(0, 2);
    lcd.print("  ");
    lcd.setCursor(12, 0);
    lcd.print("  ");
    lcd.setCursor(12, 1);
    lcd.print("  ");
  }

  if (drehen == 0)
  {
    altdrehen = 0;
    rufmodus = 19;
    lcd.setCursor(0, 0);
    lcd.print("=>");
  }

  if (drehen == 1)
  {
    altdrehen = 1;
    rufmodus = 2;
    lcd.setCursor(0, 1);
    lcd.print("=>");
  }

  if (drehen == 2)
  {
    altdrehen = 2;
    rufmodus = 40;
    lcd.setCursor(0, 2);
    lcd.print("=>");
  }

  if (drehen == 3)
  {
    altdrehen = 3;
    rufmodus = 1;
    lcd.setCursor(12, 0);
    lcd.print("=>");
  }

  if (drehen == 4)
  {
    altdrehen = 4;
    rufmodus = 5;
    lcd.setCursor(12, 1);
    lcd.print("=>");
  }

  if (ButtonPressed)
  {
    modus = rufmodus;
    if (modus == 1)
    { //Übergabe an Modus1
      drehen = 10; //ganzzahliger Vorgabewert für Kühlen
    }
    if (modus == 2)
    { //Übergabe an Modus2
      drehen = 78;                    //Nachgusstemperatur
    }
    //für Sollwert
    anfang = 0;
    lcd.clear();
  }

}
// ------------------------------------------------------------------------------------------

//Funktion SetupMenu-------------------------------------------------------------------------

void funktion_SetupMenu()      //Modus=5
{
  if (anfang == 0)

  {
    lcd.clear();
    lcd.setCursor(2, 0);
    lcd.print("Schwelle");
    lcd.setCursor(2, 1);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrer");
    lcd.setCursor(2, 2);
    lcd.print("Funk");
    lcd.setCursor(14, 0);
    lcd.print("Regler");
    lcd.setCursor(14, 1);
    lcd.print("Zur"); lcd.print(char(0xF5)); lcd.print("ck");
    drehen = 0;
    altdrehen = 0;
    anfang = 1;
  }


  if (drehen == 255)
  {
    drehen = 4;
  }
  if (drehen == 5)
  {
    drehen = 0;
  }

  if (drehen != altdrehen)
  {
    lcd.setCursor(0, 0);
    lcd.print("  ");
    lcd.setCursor(0, 1);
    lcd.print("  ");
    lcd.setCursor(0, 2);
    lcd.print("  ");
    lcd.setCursor(12, 0);
    lcd.print("  ");
    lcd.setCursor(12, 1);
    lcd.print("  ");
    altdrehen = drehen;
  }

  if (drehen == 0)
  {
    altdrehen = 0;
    rufmodus = 239;
    lcd.setCursor(0, 0);
    lcd.print("=>");
  }

  if (drehen == 1)
  {
    altdrehen = 1;
    rufmodus = 23;
    lcd.setCursor(0, 1);
    lcd.print("=>");
  }

  if (drehen == 2)
  {
    altdrehen = 2;
    rufmodus = 233;
    lcd.setCursor(0, 2);
    lcd.print("=>");
  }

  if (drehen == 3)
  {
    altdrehen = 3;
    rufmodus = 11;
    lcd.setCursor(12, 0);
    lcd.print("=>");
  }


  if (drehen == 4)
  {
    altdrehen = 4;
    rufmodus = altmodus;
    lcd.setCursor(12, 1);
    lcd.print("=>");

  }

  if (ButtonPressed)
  {

    if (dclick) {
      anfang = 1;
    } else {
      anfang = 0;
    }
    modus = rufmodus;
    drehen = tempdrehen;
    first = true;
    lcd.clear();
  }

}

// ------------------------------------------------------------------------------------------


// Funktion nur Temperaturregelung ------------------------------------------------------------
void funktion_temperatur()      //Modus=1 bzw.2
{

  if (drehen == 255) drehen = 0;
  sollwert = drehen;


  if (modus == 1)
  {
    lcd.setCursor(0, 0);
    lcd.print("K"); lcd.print(char(0xF5)); lcd.print("hlen");
    altmodus = 1; // Merker für Doppelclick
  }

  if (modus == 2)
  {
    lcd.setCursor(0, 0);
    lcd.print("Nachguss");
  }

  if ((modus == 2) && (isttemp >= sollwert) && (nachgussruf == false))   //Nachguss -> Sollwert erreicht
  {
    nachgussruf = true;
    rufmodus = modus;          //Rufalarm
    modus = 31;
    y = 0;
    braumeister[y] = 2;        //nur Ruf und weiter mit Regelung
  }

  lcd.setCursor(10, 1);
  lcd.print("soll");

}
// ------------------------------------------------------------------------------------------


// Funktion Eingabe der Rastanzahl-------------------------------------------------------------
void funktion_rastanzahl()          //Modus=19
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    lcd.setCursor(12, 1);
    lcd.print("Rasten");

    drehen = rasten;
    anfang = 1;
    dclick = false;
  }

  //Vorgabewerte bei verschiedenen Rasten
  if (rasten != drehen)
  {
    if ((int)drehen == 1)
    {
      rastTemp[1] = 67;
      rastZeit[1] = 60;
      maischtemp = 65;
    }
    if (drehen == 2)
    {
      rastTemp[1] = 62;
      rastZeit[1] = 30;
      rastTemp[2] = 72;
      rastZeit[2] = 35;
      maischtemp = 62;
    }
    if (drehen == 3)
    {
      rastTemp[1] = 55;
      rastZeit[1] = 10;
      rastTemp[2] = 62;
      rastZeit[2] = 35;
      rastTemp[3] = 72;
      rastZeit[3] = 25;
      maischtemp = 55;
    }
    if (drehen == 4)
    {
      rastTemp[1] = 40;
      rastZeit[1] = 20;
      rastTemp[2] = 55;
      rastZeit[2] = 15;
      rastTemp[3] = 64;
      rastZeit[3] = 35;
      rastTemp[4] = 72;
      rastZeit[4] = 25;
      maischtemp = 35;
    }
    if (drehen == 5)
    {
      rastTemp[1] = 35;
      rastZeit[1] = 20;
      rastTemp[2] = 40;
      rastZeit[2] = 20;
      rastTemp[3] = 55;
      rastZeit[3] = 15;
      rastTemp[4] = 64;
      rastZeit[4] = 35;
      rastTemp[5] = 72;
      rastZeit[5] = 25;
      maischtemp = 30;
    }
  }

  rasten = drehen;


  if (rasten <= 1)
  {
    rasten = 1;
    drehen = 1;
  }
  if (rasten >= 5)
  {
    rasten = 5;
    drehen = 5;
  }

  lcd.setCursor(19, 1);
  lcd.print(rasten);

  if (ButtonPressed)
  {
    modus++;
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------


// Funktion Maischtemperatur-----------------------------------------------------------------
void funktion_maischtemperatur()      //Modus=20
{

  if ((anfang == 0) || (dclick))
  {
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    drehen = maischtemp;
    anfang = 1;
    first = true;
    dclick = false;
  }

  maischtemp = drehen;

  if (maischtemp <= 10)
    maischtemp = 10;

  if (maischtemp >= 105)
    maischtemp = 105;



  lcd.setCursor(0, 1);
  lcd.print("Einmaischen         ");
  lcd.setCursor(15, 1);
  lcd.print(maischtemp);
  lcd.print(char(0xDF)); lcd.print("C ");

  if (ButtonPressed)
  {
    modus++;
    anfang = 0;
  }
}
// -------------------------------------------------------------------------------------------


// Funktion Rasteingabe Temperatur--------------------------------------------------------------
void funktion_rasteingabe()      //Modus=21
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    drehen = rastTemp[x];
    anfang = 1;
    dclick = false;
  }

  rastTemp[x] = drehen;

  if (rastTemp[x] <= 10)
  {
    rastTemp[x] = 9;
    drehen = 9;
  }
  if (rastTemp[x] >= 105)
  {
    rastTemp[x] = 105;
    drehen = 105;
  }

  lcd.setCursor(14, 0);
  lcd.print(x);
  lcd.print(".Rast");
  lcd.setCursor(15, 1);
  lcd.print(int(rastTemp[x]));
  lcd.print(char(0xDF)); lcd.print("C ");



  if (ButtonPressed)
  {
    modus++;
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------


// Funktion Rasteingabe Zeit-------------------------------------------------------------------
void funktion_zeiteingabe()      //Modus=22
{

  if ((anfang == 0) || (dclick))
  {
    drehen = rastZeit[x];
    anfang = 1;

    lcd.setCursor(0, 0);
    lcd.print("Eingabe");

    lcd.setCursor(14, 0);
    lcd.print(x);
    lcd.print(".Rast");
    lcd.setCursor(15, 1);
    lcd.print(int(rastTemp[x]));
    lcd.print(char(0xDF)); lcd.print("C ");


    dclick = false;
  }

  rastZeit[x] = drehen;

  if (rastZeit[x] <= 1)
  {
    rastZeit[x] = 1;
    drehen = 1;
  }
  if (rastZeit[x] >= 99)
  {
    rastZeit[x] = 99;
    drehen = 99;
  }

  if (rastZeit[x] < 10)
  {
    lcd.setCursor(14, 2);
    lcd.print(" ");
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }
  else
  {
    lcd.setCursor(14, 2);
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }


  if (ButtonPressed)
  {
    modus = 24;
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------


void ruehrwerk() //Modus=23 -------------------------------------------------------------------
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrwerk");
    lcd.setCursor(0, 1);
    lcd.print("Dauerbetrieb");
    anfang = 1;
    drehen = EEPROM.read(RuehrerDauerbetrieb);
    dclick = false;
  }

  char antwort = ja_nein[drehen % 2];

  lcd.setCursor(14, 1); lcd.print(antwort);

  dauerbetrieb = antwort;

  if (ButtonPressed)
  {
    if (dauerbetrieb == 'N') {
      modus = 231;
      EEPROM.write(RuehrerDauerbetrieb, 0);
    } else {
      modus = 5;
      EEPROM.write(RuehrerDauerbetrieb, 1);
    }
    speichern(2);
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Rührwerk Intervall EIN-----------------------------------------------------------
void ruehrwerk_ein()      //Modus=231
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrwerk");
    lcd.setCursor(0, 1);
    lcd.print("Intervall");
    drehen = intervalEin;
    anfang = 1;
    dclick = false;
  }

  if (drehen > 250) drehen = 0;
  intervalEin = drehen;

  if (intervalEin > 9)
  {
    lcd.setCursor(0, 1); lcd.print("Intervall EIN: "); lcd.print(intervalEin); lcd.print("s  ");
  }
  else
  {
    lcd.setCursor(0, 1); lcd.print("Intervall EIN: "); lcd.print(intervalEin); lcd.print("s   ");
  }


  if (ButtonPressed)
  {
    EEPROM.write(RuehrerIntervallEin, intervalEin);
    speichern(2);
    modus++;
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Rührwerk Intervall AUS-----------------------------------------------------------

void ruehrwerk_aus()      //Modus=232
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrwerk");
    lcd.setCursor(0, 1);
    lcd.print("Intervall");
    drehen = intervalAus;
    anfang = 1;
    dclick = false;
  }

  if (drehen > 250) drehen = 0;

  intervalAus = drehen;

  if (intervalAus > 9)
  {
    lcd.setCursor(0, 1); lcd.print("Intervall AUS: "); lcd.print(intervalAus); lcd.print("s  ");
  }
  else
  {
    lcd.setCursor(0, 1); lcd.print("Intervall AUS: "); lcd.print(intervalAus); lcd.print("s   ");
  }


  if (ButtonPressed)
  {
    EEPROM.write(RuehrerIntervallAus, intervalAus);
    speichern(2);
    modus = 5;
    anfang = 0;
  }
}

// ------------------------------------------------------------------------------------------

void steckdosencode() //Modus 233 -----------------------------------------------------------
{
  if (anfang == 0)
  {
    lcd.clear();
    mySwitch.enableReceive(0);  //477 MHz Empfänger an Interrupt 0 (Pin #2)
    anfang = 1;
    drehen = 1;
  }
  else
  {
    if (drehen == 1) {
      lcd.setCursor(0, 1);
      lcd.print("Taste Heizung EIN");
      adr = HeizungEin_433;

    }
    if (drehen == 2) {
      lcd.setCursor(0, 1);
      lcd.print("Taste Heizung AUS");
      adr = HeizungAus_433;
    }
    if (drehen == 3) {
      lcd.setCursor(0, 1);
      lcd.print("Taste Ruehrer EIN");
      adr = RuehrerEin_433;
    }
    if (drehen == 4) {
      lcd.setCursor(0, 1);
      lcd.print("Taste Ruehrer AUS");
      adr = RuehrerAus_433;
    }
    if (drehen == 5) {
      lcd.setCursor(0, 1);
      lcd.print("Taste Ruf EIN");
      adr = RufEin_433;
    }
    if (drehen == 6) {
      lcd.setCursor(0, 1);
      lcd.print("Taste Ruf AUS");
      adr = RufAus_433;
    }

    if (drehen > 6) drehen = 1;


    value = mySwitch.getReceivedValue();
    if (value) {
      // value in EPROM schreiben
      if (EEPROMReadlong(adr) != value) EEPROMWritelong(adr, value);
      lcd.setCursor(0, 2);
      lcd.print("speichern");
      delay(500);
      drehen++;
      if (drehen == 7) {
        modus = 0;
        anfang = 0;
        mySwitch.disableReceive();  //Empfänger wieder deaktivieren
      }
      lcd.clear();
    }
    mySwitch.resetAvailable();

  }
}
// ------------------------------------------------------------------------------------------

// Funktion Kochschwelle einstellen ---------------------------------------------------------

void kochschwelle()  //Modus 239
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0); lcd.print("Kochschwelle");
    lcd.setCursor(0, 1); lcd.print("Ruf bei:");
    drehen = EEPROM.read(Koch_Schwelle); //EPROM-Speicher auslesen
    anfang = 1;
    dclick = false;
  }
  else
  {


    if (drehen > 99) drehen = 20;
    if (drehen <= 20) drehen = 99;

    kschwelle = drehen;


    lcd.setCursor(9, 1); lcd.print(kschwelle);


    if (ButtonPressed)
    {
      EEPROM.write(Koch_Schwelle, kschwelle);
      speichern(2);
      modus = 5;
      anfang = 0;
    }
  }
}
// ------------------------------------------------------------------------------------------


// Funktion Intervallbetrieb-----------------------------------------------------------------
void intervallbetrieb()
{
  if (regelung == 1)
  {
    unsigned long ein = 1000 * (unsigned long)intervalEin;
    unsigned long aus = 1000 * (unsigned long)intervalAus;

    if (firstinterval)
    {
      intervalstart = millis(); firstinterval = false;
    }
    else if ((intervalstart  + ein)  >= millis()) {
      if (rwerk == 0) ruehrerEinAus(1);
    }
    else if ((intervalstart  + ein + aus) >= millis()) {
      if (rwerk == 1) ruehrerEinAus(0);
    }
    else firstinterval = true;
  }
}
// ------------------------------------------------------------------------------------------

void ruehrerEinAus(int ruehrer)
{
  if ((regelung == 1) && (modus != 2) && (modus != 1)) // außer bei Nachgußbereitung

    if (ruehrer == 1)
    {
      if (rwerk == 0)
      {
        delay(100); //wg. Funksignal (Zeitabstand zwischen 2 Funksignalen (Heizung und Rührer)
        mySwitch.send(funk_ruehrer_ein, 24);

        rwerk = 1;
        delay(100); //wg. Funksignal
      }


    }
  if (ruehrer == 0)
  {
    if (rwerk == 1)
    {
      delay(100); //wg. Funksignal
      mySwitch.send(funk_ruehrer_aus, 24);

      rwerk = 0;
      delay(100); //wg. Funksignal
    }
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Braumeister----------------------------------------------------------------------
void funktion_braumeister() //Modus=24
{

  braumeister[x] = 0;
  if (x < rasten)
  {
    x++;
    modus = 21;           //Sprung zur Rasttemperatureingabe
    anfang = 0;
  }
  else
  {
    x = 1;
    modus++;             //Sprung zur Rastzeiteingabe
    anfang = 0;
  }
}
//--------------------------------------------------------------------------------------------

// Funktion Ende Temperatur-------------------------------------------------------------------
void funktion_endtempeingabe()      //Modus=25
{

  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    drehen = endtemp;
    anfang = 1;
  }

  endtemp = drehen;

  if (endtemp <= 10)
  {
    endtemp = 10;
    drehen = 10;
  }

  if (endtemp > 80)
  {
    endtemp = 80;
    drehen = 80;
  }


  lcd.setCursor(0, 1);
  lcd.print("Abmaischen");
  lcd.setCursor(15, 1);
  lcd.print(int(endtemp));
  lcd.print(char(0xDF)); lcd.print("C ");

  if (ButtonPressed)
  {
    modus++;
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Startabfrage---------------------------------------------------------------------
void funktion_startabfrage()      //Modus=26
{
  if (anfang == 0)
  {
    lcd.clear();
    anfang = 1;
    altsekunden = millis();
  }

  if (millis() >= (altsekunden + 1000))
  {
    lcd.setCursor(13, 1);
    lcd.print("       ");
    if (millis() >= (altsekunden + 1500))
      altsekunden = millis();
  }
  else
  {
    lcd.setCursor(13, 1);
    lcd.print("Start?");
  }

  if (ButtonPressed)
  {
    anfang = 0;
    modus++;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Automatik Maischtemperatur-------------------------------------------------------
void funktion_maischtemperaturautomatik()      //Modus=27
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Maischen Einmaischen");
    drehen = maischtemp;    //Zuordnung Encoder
    anfang = 1;
    dclick = false;
  }



  maischtemp = drehen;

  sollwert = maischtemp;

  if (isttemp >= sollwert)  // Sollwert erreicht ?
  {
    altmodus = modus;
    modus++;
    rufmodus = modus;
    y = 0;
    braumeister[y] = 1;
    modus = 31;
  }
}
//--------------------------------------------------------------------------------------------

// Funktion Automatik Temperatur--------------------------------------------------------------
void funktion_tempautomatik()      //Modus=28
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(14, 0);
    lcd.print(x);
    lcd.print(".Rast");

    firstinterval = true;  //Intervallbetrieb Rührwerk
    rwerk = 0;

    drehen = rastTemp[x];
    anfang = 1;
    first = true;
    dclick = false;
  }

  rastTemp[x] = drehen;

  sollwert = rastTemp[x];


  if (isttemp >= sollwert)  // Sollwert erreicht ?
  {
    modus++;                //zur Zeitautomatik
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Automatik Zeit-------------------------------------------------------------------
void funktion_zeitautomatik()      //Modus=29
{
  if (anfang == 0)
  {
    drehen = rastZeit[x];              //Zuordnung für Encoder
  }

  lcd.setCursor(14, 0);
  lcd.print(x);
  lcd.print(".Rast");

  if (rastZeit[x] < 10)
  {
    lcd.setCursor(14, 2);
    lcd.print(" ");
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }
  else
  {
    lcd.setCursor(14, 2);
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }


  // Zeitzählung---------------
  if (anfang == 0)
  {
    if (zeitz) setTime(00, 00, 00, 00, 01, 01); //.........Sekunden auf 0 stellen

    sekunden = second();  //aktuell Sekunde abspeichern für die Zeitrechnung
    minutenwert = minute(); //aktuell Minute abspeichern für die Zeitrechnung
    stunden = hour();     //aktuell Stunde abspeichern für die Zeitrechnung

    lcd.setCursor(0, 2);
    lcd.print("        ");
    anfang = 1;
  }

  lcd.setCursor(0, 2);

  if (sekunden < 10)
  {
    if (minuten == 0)
    { lcd.setCursor(1, 2);
      lcd.print("00:0"); lcd.print(sekunden);
    }
  }
  else
  {
    if (minuten == 0)
    { lcd.setCursor(1, 2);
      lcd.print("00:"); lcd.print(sekunden);
    }
  }


  if (stunden == 0)
    minuten = minutenwert;
  else
    minuten = ((stunden * 60) + minutenwert);

  if ((minuten < 10) && (minuten > 0))
  {
    lcd.setCursor(1, 2);
    lcd.print("0"); lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if ((minuten >= 10) && (minuten < 100))
  {
    lcd.setCursor(1, 2);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if (minuten >= 100)
  {
    lcd.setCursor(0, 2);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  //Erinnerung Jodprobe-------------------------------------------------------------

  if (x == rasten)
  {
    jodprobe = false;
    if ((minuten >= rastZeit[x] - 5) && (!signalende)) //5 min. vor Ende der letzten Rast
    {
      lcd.setCursor(0, 3); lcd.print("Jodprobe!           ");
      for (tz = 0; tz <= 5; tz++)signalton(500); delay(500);
      signalende = true;
      jodprobe = true;
    }

    if (jodprobe) {
      lcd.setCursor(0, 3);
      lcd.print("                    ");
      jodprobe = false;
    }
  }

  //--------------------------------------------------------------------------------

  // Ende Zeitzählung---------------------

  rastZeit[x] = drehen;   //Encoderzuordnung

  if (minuten >= rastZeit[x])  // Sollwert erreicht ?
  {
    anfang = 0;
    y = x;
    if (x < rasten)
    {
      modus--;                // zur Temperaturregelung
      x++;                    // nächste Stufe
    }
    else
    {
      x = 1;                              //Endtemperatur
      modus++;                            //Endtemperatur
    }

    if (braumeister[y] > 0)
    {
      rufmodus = modus;
      modus = 31;
    }
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Automatik Endtemperatur----------------------------------------------------------
void funktion_endtempautomatik()      //Modus=30
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(10, 0);
    lcd.print("Abmaischen");

    drehen = endtemp;    //Zuordnung Encoder
    anfang = 1;
    dclick = false;
  }

  endtemp = drehen;

  sollwert = endtemp;

  if (isttemp >= sollwert)  // Sollwert erreicht ?
  {
    modus = 80;                //Abbruch
    rufmodus = modus;
    modus = 31;
    regelung = 0;              //Regelung aus
    heizung = 0;               //Heizung aus
    y = 0;
    braumeister[y] = 1;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion braumeisterrufalarm--------------------------------------------------------------
void funktion_braumeisterrufalarm()      //Modus=31
{
  if (anfang == 0)
  {
    rufsignalzeit = millis();
    altsekunden = millis();
    anfang = 1;
  }

  if (altmodus == 27) //Einmaischtemperatur erreicht ! Heizung und Rührer aus bis weiter!
  {
    //   altmodus = 31;
    heizung = 0;
#ifdef RuehrerMaischenAus
    rwerk = 1;
    ruehrerEinAus(0);
#endif
  }

  if (millis() >= (altsekunden + 1500)) //Blinken der Anzeige und RUF
  {
    lcd.setCursor(17, 2);
    lcd.print("   ");
    digitalWrite(schalterB, LOW);
    mySwitch.send(funk_ruf_aus, 24); //Schaltet die Funksteckdose aus
    delay(100);
    if (millis() >= (altsekunden + 1500))
    {
      altsekunden = millis();
      pause++;
    }
  }
  else
  {
    lcd.setCursor(17, 2);
    lcd.print("Ruf");
    if (pause <= 4)
    {
      digitalWrite(schalterB, HIGH);
      mySwitch.send(funk_ruf_ein, 24); //Schaltet die Funksteckdose ein
      delay(100);
    }
    if (pause > 8)
      pause = 0;
  }                                   //Blinken der Anzeige und RUF

  //20 Sekunden Rufsignalisierung wenn "Ruf Signal"
  if (braumeister[y] == 2 && millis() >= (rufsignalzeit + 20000))
  {
    anfang = 0;
    pause = 0;
    digitalWrite(schalterB, LOW);   // Alarm ausschalten
    mySwitch.send(funk_ruf_aus, 24); //Schaltet die Funksteckdose aus
    delay(100);
    modus = rufmodus;
    lcd.setCursor(17, 2);
    lcd.print("   ");
  }
  //weiter mit Programmablauf

  if (ButtonPressed)
  {
    pause = 0;
    anfang = 0;
    digitalWrite(schalterB, LOW);   // Alarm ausschalten
    mySwitch.send(funk_ruf_aus, 24); //Schaltet die Funksteckdose aus
    delay(100);
    if (braumeister[y] == 2)
    {
      lcd.setCursor(17, 2);
      lcd.print("   ");
      if (sensorfehler == 1) {
#ifdef emcSchutz
        emccounter = emc;
#endif
        sensorfehler = 0;  //Sensorfehler oder Alarmtest
        lcd.setCursor(8, 0);
        lcd.print("            ");
      }
    }
    else
      modus++;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion braumeisterruf-------------------------------------------------------------------
void funktion_braumeisterruf()      //Modus=32
{
  if (anfang == 0)
  {
    anfang = 1;
    altsekunden = millis();
  }


  if (millis() >= (altsekunden + 1500))
  {
    lcd.setCursor(12, 2);
    lcd.print("        ");
    if (millis() >= (altsekunden + 1500))
      altsekunden = millis();
  }
  else
  {
    lcd.setCursor(0, 2); lcd.print("            ");
    lcd.setCursor(12, 2);
    lcd.print("weiter ?");
  }

  if (ButtonPressed)
  {
    lcd.setCursor(12, 2);
    lcd.print("        ");  //Text "weiter ?" löschen

    lcd.setCursor(0, 3);
    lcd.print("             "); //Löscht Text bei
    if (sensorfehler == 1) {
      sensorfehler = 0;  //Sensorfehler oder Alarmtest
      lcd.setCursor(8, 0);
      lcd.print("            ");
    }

    anfang = 0;
    modus = rufmodus;
    if (rufmodus == 42) {
      anfang = 1;   // Hopfen zugegeben!
      lcd.setCursor(0, 2);
      lcd.print("                    ");
    }

    delay(500);     //kurze Wartezeit, damit nicht
    //durch unbeabsichtigtes Drehen
    //der nächste Vorgabewert
    //verstellt wird
  }
}
// ------------------------------------------------------------------------------------------

void anzahl_hopfengaben() //Modus=38 --------------------------------------------------------
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Hopfengaben");
    lcd.setCursor(5, 1);
    lcd.print("Anzahl");
    anfang = 1;
    hanz = 3;
    drehen = hanz;
    lcd.setCursor(14, 1);
    lcd.print(hanz);
    dclick = false;
  }


  if (hanz < 9)
  {
    lcd.setCursor(14, 1); lcd.print(hanz);
  }
  else
  {
    lcd.setCursor(13, 1); lcd.print(hanz);
  }

  hanz = drehen;

  if (drehen >= maxgaben)
  {
    hanz = maxgaben; drehen = maxgaben;
  }

  if (drehen <  2)
  {
    hanz = 1; drehen = 1;
  }

  if (ButtonPressed)
  {
    modus = 39;
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Hopfengabe-----------------------------------------------------------------------
void funktion_hopfengabe()      //Modus=39
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Hopfengabe:"); //lcd.print(h + 1);
    lcd.setCursor(0, 1);
    lcd.print("Kochen ");
    lcd.print("f"); lcd.print(char(0xF5)); lcd.print("r");


    if (kochzeit >= 60)  // weniger ist eigentlich nicht sinnvoll
    {
      if (h == 0) hopfengabe[h] = kochzeit;  //Typische Zeiten für das Hopfenkochen
      if (h == 1) hopfengabe[h] = kochzeit - 20;
      if (h == 2) hopfengabe[h] = 10;
    }

    drehen = hopfengabe[h];

    anfang = 1;
    dclick = false;
  }

  lcd.setCursor(11, 0); lcd.print(h + 1);

  hopfengabe[h] = drehen;
  lcd.setCursor(12, 1);

  lcd.print(drehen);
  lcd.print(" min ");

  if (hopfengabe[h] <= 0)
  {
    hopfengabe[h] = 0;
    drehen = 0;
  }
  if (hopfengabe[h] >= kochzeit)
  {
    hopfengabe[h] = kochzeit;
    drehen = kochzeit;
  }


  if (ButtonPressed)
  {
    if (h == (hanz - 1)) {
      lcd.clear();
      modus = 41;

    } else {
      h++;
    }
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Kochzeit-------------------------------------------------------------------------
void funktion_kochzeit()      //Modus=40
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Aufheizen");
    lcd.setCursor(4, 1);
    lcd.print("Kochzeit");
    lcd.setCursor(17, 1);
    lcd.print("min");

    lcd.setCursor(2, 2);
    lcd.print("Dr"); lcd.print(char(0xF5)); lcd.print("cken = weiter");

    drehen = kochzeit;

    heizung = 1; //Heizung einschalten
    rwerk = 1;
    ruehrerEinAus(0);

    anfang = 1;
    dclick = false;
  }

  kochzeit = drehen; //5min-Sprünge

  if (kochzeit <= 20)
  {
    kochzeit = 20;
    drehen = 20;
  }
  if (kochzeit >= 180)
  {
    kochzeit = 180;
    drehen = 180;
  }

  if (kochzeit < 100)
  {
    lcd.setCursor(12, 1);
    lcd.print("  ");
    lcd.print(kochzeit);
  }
  else
  {
    lcd.setCursor(12, 1);
    lcd.print(" ");
    lcd.print(kochzeit);
  }

  if (ButtonPressed)
  {
    modus = 38; // Anzahl Hopfengaben
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Startkochzeit--------------------------------------------------------------------
void funktion_startkochen()      //Modus=41
{
  if ((anfang == 0) || (dclick))
  {
    anfang = 1;
    altsekunden = millis();
    abbruchkochen = 60;
    blinker = 0;
    dclick = false;
  }
  lcd.setCursor(0, 0); lcd.print("Ruf bei "); lcd.print(kschwelle); lcd.print(char(0xDF)); lcd.print("C"); lcd.print("  ");
  if (millis() <= (altsekunden + 1000))
  {

    if ((isttemp >= kschwelle) || (schwelle_erreicht))
    {
      schwelle_erreicht = true;
      if (anfang != 3) {
        lcd.setCursor(0, 1);
        lcd.print("Abbruch in ");
        if (abbruchkochen <= 60) {
          lcd.print(abbruchkochen);
          lcd.print(" s ");
        }
      }
      lcd.setCursor(0, 2);
      if (blinker == 0)
      {
        if (anfang == 3)
        {
          lcd.print("Start Zeitz");
          lcd.print(char(0xE1));
          lcd.print("hlung?");
        }
        else
        {
          lcd.print("Bitte quittieren!");
        }
        if (anfang == 1) {
          signalton(500);
          anfang = 2;
        } else if (anfang == 2)
        { if (abbruchkochen > 61) modus = 80;
          if (abbruchkochen % 10 == 0) signalton(500);
        }
      }
      else lcd.print("                   ");

    }
    else
    {
      if (blinker == 0)
      {
        lcd.setCursor(0, 2);
        lcd.print("Warten!");
        schwelle_erreicht = false;
      }

      if (blinker == 1)
      {
        lcd.setCursor(0, 2);
        lcd.print("                   ");
      }
    }

  } else {
    if (blinker == 0) blinker = 1;
    else blinker = 0;
    altsekunden = millis();
    if (schwelle_erreicht) abbruchkochen--;
  }

  if (schwelle_erreicht)
  {
    if (ButtonPressed)
    {
      if (anfang == 3)
      {
        anfang = 0;
        lcd.setCursor(0, 2); lcd.print("               ");
        modus = 42;
      }
      if (anfang == 2) {
        anfang = 3;
        lcd.setCursor(0, 1);
        lcd.print("                    ");
      }

    }
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Kochen---------------------------------------------------------------------------
void funktion_kochen()      //Modus=42
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Kochen");
    lcd.setCursor(4, 1);
    lcd.print("Kochzeit");
    lcd.setCursor(17, 1);
    lcd.print("min");

    if (!dclick) setTime(00, 00, 00, 00, 01, 01); //.........Sekunden auf 0 stellen

    delay(400); //Anzeigepause

    sekunden = second();  //aktuell Sekunde abspeichern für die Zeitrechnung
    minutenwert = minute(); //aktuell Minute abspeichern für die Zeitrechnung
    stunden = hour();     //aktuell Stunde abspeichern für die Zeitrechnung

    anfang = 1;
    if (!dclick) h = 0;  //Hopfengabengabe Zähler
    drehen = kochzeit;
    dclick = false;
  }

  if (kochzeit < 100)
  {
    lcd.setCursor(12, 1);
    lcd.print("  ");
    lcd.print(kochzeit);
  }
  else
  {
    lcd.setCursor(12, 1);
    lcd.print(" ");
    lcd.print(kochzeit);
  }



  lcd.setCursor(14, 0);


  if (sekunden < 10)
  {
    if (minuten == 0)
    { lcd.setCursor(14, 0);
      lcd.print("00:0"); lcd.print(sekunden);
    }
  }
  else
  {
    if (minuten == 0)
    { lcd.setCursor(14, 0);
      lcd.print("00:"); lcd.print(sekunden);
    }
  }

  minuten = ((stunden * 60) + minutenwert);

  if ((kochzeit - minuten == hopfengabe[h]) && (h < hanz))
  {
    lcd.setCursor(0, 2); lcd.print(h + 1); lcd.print(". H-Gabe");
    h++;
    signalton(2000);
  }

  if ((sekunden + 10) % 30 == 0) {
    lcd.setCursor(0, 2);
    lcd.print("          ");
  }

  if ((minuten < 10) && (minuten > 0))
  {
    lcd.setCursor(14, 0);
    lcd.print("0"); lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if ((minuten >= 10) && (minuten < 100))
  {
    lcd.setCursor(14, 0);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if (minuten >= 100)
  {
    lcd.setCursor(13, 0);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  kochzeit = drehen; //5min-Sprünge

  if (kochzeit <= 20)
  {
    kochzeit = 20;
    drehen = 20;
  }
  if (kochzeit >= 180)
  {
    kochzeit = 180;
    drehen = 180;
  }

  if (minuten >= kochzeit)     //Kochzeitende
  {
    modus = 80;                //Abbruch nach Rufalarm
    rufmodus = modus;
    modus = 31;
    regelung = 0;              //Regelung aus
    heizung = 0;               //Heizung aus
    y = 0;
    braumeister[y] = 2;
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------

void kDHWert() //Modus=11 -------------------------------------------------------------------
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("k(H) = ");

    kh =  EEPROMReadlong(Gradienten_Faktor_Heizen );
    anfang = 1;
    drehen = kh;
    dclick = false;
  }

  if (drehen == 255) drehen = 200;
  if (drehen > 200) drehen = 0;


  kh  = drehen;
  lcd.setCursor(10, 0); lcd.print(kh / 10); lcd.print(' ');


  if (ButtonPressed)
  {

    if (EEPROMReadlong(Gradienten_Faktor_Heizen ) != drehen) EEPROMWritelong(Gradienten_Faktor_Heizen , drehen);
    speichern(2);
    modus = 12;
    kh /= 10;
    anfang = 0;
  }
}

// ------------------------------------------------------------------------------------------

void ESVHeiz() //Modus=12 -------------------------------------------------------------------
{

  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("ESV(H) = ");

    esvheizen =  (EEPROMReadlong(Einschaltverzoegerung_Heizung));
    anfang = 1;
    drehen = esvheizen / 1000;
  }

  if (drehen == 255) drehen = 60;
  if (drehen > 60) drehen = 0;

  lcd.setCursor(10, 0); lcd.print(drehen); lcd.print(" s ");

  esvheizen = drehen * 1000L;

  if (ButtonPressed)
  {
    if (EEPROMReadlong(94) != esvheizen) EEPROMWritelong(Einschaltverzoegerung_Heizung, esvheizen);

    speichern(2);
    modus = 13;
    anfang = 0;
  }

}
// ------------------------------------------------------------------------------------------

void kDKWert() //Modus=13 -------------------------------------------------------------------
{

  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("k(K) = ");

    kk =  EEPROMReadlong(Gradienten_Faktor_Kuehlen);
    anfang = 1;
    drehen = kk;
  }

  if (drehen == 255) drehen = 200;
  if (drehen > 200) drehen = 0;


  kk  = drehen;
  lcd.setCursor(10, 0); lcd.print(kk / 10); lcd.print(' ');


  if (ButtonPressed)
  {
    if (EEPROMReadlong(Gradienten_Faktor_Kuehlen) != drehen) EEPROMWritelong(Gradienten_Faktor_Kuehlen, drehen);
    speichern(2);
    modus = 14;
    kk /= 10;
    anfang = 0;
  }
}
// ------------------------------------------------------------------------------------------

// Funktion Einschaltverzögerung Kühlen (Kompressorschutz)-----------------------------------

void wartenKuehlen()  //Modus 14
{
  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0); lcd.print("Wartezeit K"); lcd.print(char(0xF5)); lcd.print("hlen");
    kzeitverz = EEPROMReadlong(Kuehlzeit_Verzoegerung);
    kzeitverz = kzeitverz / 60000;
    drehen = kzeitverz;
    anfang = 1;
  }
  else
  {

    kzeitverz = drehen;


    lcd.setCursor(0, 1); lcd.print(kzeitverz); lcd.print(" min  ");

    if (ButtonPressed)
    {
      EEPROMWritelong(Kuehlzeit_Verzoegerung, kzeitverz * 60000);
      kzeitverz = kzeitverz * 60000;
      speichern(2);
      modus = 5;
      anfang = 0;
    }
  }
}
// ------------------------------------------------------------------------------------------


void signalton(int dauer)
{
  digitalWrite(schalterB, HIGH);   // Alarm einschalten
  mySwitch.send(funk_ruf_ein, 24); //Schaltet die Funksteckdose ein
  delay(dauer);
  digitalWrite(schalterB, LOW);   // Alarm ausschalten
  mySwitch.send(funk_ruf_aus, 24); //Schaltet die Funksteckdose aus
  delay(100);
}

// ------------------------------------------------------------------------------------------
void EEPROMWritelong(int address, long value)
{
  byte four = (value & 0xFF);
  byte three = ((value >> 8) & 0xFF);
  byte two = ((value >> 16) & 0xFF);

  EEPROM.write(address, four);
  EEPROM.write(address + 1, three);
  EEPROM.write(address + 2, two);
}

// ------------------------------------------------------------------------------------------
long EEPROMReadlong(long address)
{
  long four = EEPROM.read(address);
  long three = EEPROM.read(address + 1);
  long two = EEPROM.read(address + 2);

  return ((four << 0) & 0xFF) + ((three << 8) & 0xFFFF) + ((two << 16) & 0xFFFFFF);
}

// ------------------------------------------------------------------------------------------
void init_funk()
{
  funk_heizung_ein = EEPROMReadlong(HeizungEin_433);
  funk_heizung_aus = EEPROMReadlong(HeizungAus_433);
  funk_ruehrer_ein = EEPROMReadlong(RuehrerEin_433);
  funk_ruehrer_aus = EEPROMReadlong(RuehrerAus_433);
  funk_ruf_ein = EEPROMReadlong(RufEin_433);
  funk_ruf_aus = EEPROMReadlong(RufAus_433);
}

// ------------------------------------------------------------------------------------------
void speichern(byte zeile)
{
  lcd.setCursor(0, zeile);
  lcd.print("speichern");
  delay(500);
}


// Funktion Abbruch--------------------------------------------------------------------------
void funktion_abbruch()       // Modus 80
{
  regelung = 0;
  heizung = 0;
  sollwerterreicht = false;
  mySwitch.send(funk_heizung_aus, 24); //Schaltet die FUnksteckdose aus
  delay(100);
  if (rwerk == 1) ruehrerEinAus(0);
  digitalWrite(schalterB, LOW);   // ausschalten
  mySwitch.send(funk_ruf_aus, 24); //Schaltet die Funksteckdose aus

  sensorfehler = 0;

  lcd.clear();

  x = 1;                             //Rasteingaben
  h = 0;                             //Hopfeneingaben
  y = 1;
  n = 0;
  nachgussruf = false;
  pause = 0;
  drehen = sollwert;                //Zuweisung für Funktion Temperaturregelung

  schwelle_erreicht = false;        // für Kochbeginn Meldung

  zeitz = true;                     // Zeit läuft nach Doppelclick weiter wenn zeitz = false

  first = true;                     //Zeitzälung Kühlen und Heizen nicht beim 1. Anlauf

  kzeitverz = 0;                    //Kochzeitverzögerung
  adr = 1;                          //Adresse Funksteckdosen im EEPROM

  eingeschaltet = false;            //Kontrolle Funkschalter
  anfang = 0;                       //Daten zurücksetzen
  rufmodus = 0;                     //bleiben erhalten
  altmodus = 0;                     //Merker Doppelklick wird gelöscht
  modus = 0;                        //Hauptmenue
}
// ------------------------------------------------------------------------------------------
//Ende des Programms
// ------------------------------------------------------------------------------------------



